package com.winhc.kafka.consumer;

import com.winhc.common.enums.CompanyEnum;
import com.winhc.service.RelationService;
import com.winhc.utils.CompanyUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.context.annotation.Bean;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.ConsumerAwareListenerErrorHandler;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @author π
 * @Description:案件增量消费
 * @date 2021/5/17 16:12
 */
@Slf4j
@Service
@AllArgsConstructor
public class KafkaConsumerCaseMerge {

    private final Map<String, RelationService> map;

    @KafkaListener(id = "${spring.kafka.topic_case}"
            , topics = "${spring.kafka.topic_case}"
            , groupId = "${spring.kafka.consumer.group-id}", containerFactory = "containerFactory", errorHandler = "KafkaConsumerCaseMerge")
    public void consumerCompanyNode(List<ConsumerRecord<?, ?>> records) {
        List<Map<String, Object>> listMap = CompanyUtils.map(records);
        this.map.get(CompanyEnum.TopicType.CASE_RELATION.VALUE).save(CompanyUtils.filterList(listMap, CompanyEnum.TopicType.CASE_RELATION.CODE));
        this.map.get(CompanyEnum.TopicType.CASE_NODE.VALUE).save(CompanyUtils.filterList(listMap, CompanyEnum.TopicType.CASE_NODE.CODE));
    }

    /**
     * 因为手动确认消费，若消费失败，记录重刷
     */
    @Bean("KafkaConsumerCaseMerge")
    public ConsumerAwareListenerErrorHandler dealError() {
        return (message, e, consumer) -> {
            System.out.println(e.toString());
            System.out.println(message);
            return null;
        };
    }
}
